@@ -2,7 +2,7 @@ package Geo::Distance;
 use 5.008001;
 use strict;
 use warnings;
-our $VERSION = '0.22';
+our $VERSION = '0.23';
 
 =encoding utf8
 
@@ -44,6 +44,10 @@ calculations.
 This module has been gutted and is now a wrapper around L<GIS::Distance>, please
 use that module instead.
 
+When switching from this module to L<GIS::Distance> make sure you reverse the
+coordinates when passing them to L<GIS::Distance/distance>.  GIS::Distance takes
+lat/lon pairs while Geo::Distance takes lon/lat pairs.
+
 =head1 STABILITY
 
 The interface to Geo::Distance is fairly stable nowadays.  If this changes it 
@@ -62,6 +66,37 @@ C<0.07> - OO only, and other changes all over.
 use GIS::Distance;
 use GIS::Distance::Constants qw( :all );
 use Carp qw( croak );
+use Const::Fast;
+
+=head1 FORMULAS
+
+C<cos> - See L<GIS::Distance::Cosine>.
+
+C<gcd> - See L<GIS::Distance::GreatCircle>.
+
+C<hsin> - See L<GIS::Distance::Haversine>.
+
+C<mt> - See L<GIS::Distance::MathTrig>.
+
+C<null> - See L<GIS::Distance::Null>.
+
+C<polar> - See L<GIS::Distance::Polar>.
+
+C<tv> - See L<GIS::Distance::Vincenty>.
+
+=cut
+
+const our %GEO_TO_GIS_FORMULA_MAP => (qw(
+    cos   Cosine
+    gcd   GreatCircle
+    hsin  Haversine
+    mt    MathTrig
+    null  Null
+    polar Polar
+    tv    Vincenty
+));
+
+const our @FORMULAS => (keys %GEO_TO_GIS_FORMULA_MAP);
 
 =head1 PROPERTIES
 
@@ -147,9 +182,10 @@ sub new {
     $geo->formula('cos');
 
 Allows you to retrieve and set the formula that is currently being used to
-calculate distances.  The available formulas are hsin, polar, cos and mt. hsin
-is the default and mt/cos are deprecated in favor of hsin. Polar should be used
-when calculating coordinates near the poles.
+calculate distances.  See the available L</FORMULAS>.
+
+C<hsin> is the default.  Both C<mt> and C<cos> are inferior in speed
+and accuracy to C<hsin>.
 
 =cut
 
@@ -159,16 +195,13 @@ sub formula {
     return $self->{formula} if !$_[0];
     my $formula = shift;
 
-    my $gis_formula = ($formula eq 'mt')    ? 'MathTrig'
-                    : ($formula eq 'cos')   ? 'Cosine'
-                    : ($formula eq 'hsin')  ? 'Haversine'
-                    : ($formula eq 'polar') ? 'Polar'
-                    : ($formula eq 'gcd')   ? 'GreatCircle'
-                    : ($formula eq 'tv')    ? 'Vincenty'
-                    : undef;
+    my $gis_formula = $GEO_TO_GIS_FORMULA_MAP{ $formula };
 
-    croak('Invalid formula (only mt, cos, hsin, polar, gcd and tv are supported)')
-        if !$gis_formula;
+    croak(
+        'Unknown formula (available formulas are ',
+        join(', ', sort @FORMULAS),
+        ')',
+    ) if !$gis_formula;
 
     $self->{formula} = $formula;
     $self->{gis_formula} = $gis_formula;
@@ -245,17 +278,21 @@ sub distance {
     croak('Unkown unit type "' . $unit . '"') if !$unit_rho;
 
     my $gis = GIS::Distance->new( $self->{gis_formula} );
-    my $km = $gis->{code}->( $lon1, $lat1, $lon2, $lat2 );
+
+    # Reverse lon/lat to lat/lon, the way GIS::Distance wants it.
+    my $km = $gis->{code}->( $lat1, $lon1, $lat2, $lon2 );
 
     return $km * ($unit_rho / $KILOMETER_RHO);
 }
 
-use Math::Trig qw( pi deg2rad );
+use Math::Trig qw( acos asin atan deg2rad great_circle_distance pi tan );
 
 sub old_distance {
     my($self,$unit,$lon1,$lat1,$lon2,$lat2) = @_;
     croak('Unkown unit type "'.$unit.'"') unless($unit = $self->{units}->{$unit});
 
+    return 0 if $self->{formula} eq 'null';
+
     if($self->{formula} eq 'mt'){
         return great_circle_distance(
             deg2rad($lon1),
@@ -457,20 +494,6 @@ sub closest {
 1;
 __END__
 
-=head1 FORMULAS
-
-C<cos> - See L<GIS::Distance::Cosine>.
-
-C<gcd> - See L<GIS::Distance::GreatCircle>.
-
-C<hsin> - See L<GIS::Distance::Haversine>.
-
-C<mt> - See L<GIS::Distance::MathTrig>.
-
-C<polar> - See L<GIS::Distance::Polar>.
-
-C<tv> - See L<GIS::Distance::Vincenty>.
-
 =head1 AUTHORS
 
     Aran Clary Deltac <bluefeet@cpan.org>
